/*******************************************************************************
 * WJ Communications - Distributed Autonomous Reader Network
 * Zigbee host firmware
 * HostMain.c - Main program
 *
 * Author: Michael Walter-Echols
 * 
 * (C) WJCI 2006
 * 401 River Oaks Pkwy
 * San Jose, CA 95134-1916
 * USA
 ******************************************************************************/

#include "mcuinit.h"								// for Init_Device()
#include "portdefs.h"								// LEDs and buttons
#include "ZigbeeHost.h"								// network and packet functions
#include "serial.h"									// serial communications functions
#include "../include/HS_net/HS_Net.h"				// Zigbee stack

#define LED_TIMEOUT 10000							// number of loop cycles to light LED

extern buffer xdata uart_buf;						// UART1 serial buffer
extern BYTE xdata z_buf[Z_BUF_SIZE];				// Zigbee transmit buffer
extern BYTE xdata nodeAddr;

void main(void) {
	UINT8	pktLength;
	UINT8	bytesPending;							// bytes pending Zigbee transmission
	UINT16	rxLedCnt = 0;							// receive LED indicator counter
	UINT16	txLedCnt = 0;							// transmit LED indicator counter
	BOOL	xdata started = FALSE;
	BOOL	xdata canSend = TRUE;
	BOOL	xdata dataReceived = FALSE;				// data received by Zigbee flag
	BOOL	xdata dataPending = FALSE;				// data pending Zigbee transmission 
	NLDE_DATA_INDICATION xdata *zDataInd;
	BYTE	xdata cobank;							// code bank

	
	/* Silicon Labs' initialization sequence
	 *
	 * This sets up devices on the crossbar, as well as initializes the RF tranceiver
	 * and Zigbee stack
	 */
	InitAllSystem();

	/* Our initialization sequence
	 *
	 * This reconfigures the crossbar for our purposes (disables SMBus and enables
	 * UART1 on end devices) and configures oscillators and interrupts for UARTs
	 */
	Init_Device();	

	cobank = PSBANK;								// backup current code bank
	PSBANK |= 0x30;									// switch to bank 3
	nodeAddr = *(BYTE code *)(NODE_ADDRESS);		// retrieve node address
	PSBANK = cobank;								// restore code bank

	buf_init(&uart_buf);							// initialize serial buffer
	nlmeConfirm.confirmId = 0;						// reset confirmation interface

	while (!BUTTON_SW6)
	{
		/* Simple counter-based timer to keep indicator LEDs lit long enough to see */
		if (rxLedCnt && !(--rxLedCnt))
		{
			LED12_OFF();
		}
		if (txLedCnt&& !(--txLedCnt))
		{
			LED13_OFF();
		}
		/* This does magical non-realtime Zigbee stuff */
		LoopProce();								// feed the Zigbee

		if (!started)
		{
		/* Start in the appropriate mode based on user button selection */
			if (BUTTON_SW3)
			{
				CreateZigbeeCoordinator();
				started = TRUE;
			}
			if (BUTTON_SW4)
			{
				CreateZigbeeRouter();
				started = TRUE;
			}
			if (BUTTON_SW5)
			{
				CreateZigbeeEndDevice();
				started = TRUE;
			}
		}
		/* Control basic operation once we are networked */
		else
		{
			/* Set state control flags based on network indications and confirmations */
			switch (nlmeConfirm.confirmId)
			{
			case N_DATA_CFM:						// if data confirmation
				canSend = TRUE;						// allow sending
				break;
			case N_DATA_IND:						// if data indication
				dataReceived = TRUE;				// set data received flag
				zDataInd = (NLDE_DATA_INDICATION *)nlmeConfirm.buffer;
													// point to indication
				break;
			default:;
			}
			nlmeConfirm.confirmId = 0;
			/* This block handles data received over the Zigbee link.
			 *
			 * If the packet contains a local command, the command is executed.
			 * If the packet contains a controller command, the packet is forwarded
			 * to the connected reader.
			 */
			if (dataReceived)
			{
				packet xdata *pktPtr = (packet xdata *)zDataInd->pNsdu;
				LoopProce();						// feed the Zigbee beast
				dataReceived = FALSE;				// clear data received flag
				
				if (pktPtr->payload[CMD] == LOCAL_CMD && crc_ok(pktPtr))
				{									// if local command
					switch (pktPtr->payload[SUBCMD])
					{
					case ECHO:						// return ping response
						pktPtr = (packet xdata *)z_buf;
						packet_init(pktPtr);
						pktPtr->payload[STATUS] = 0;
						pktPtr->len++;
						packet_calc_crc(pktPtr);
						bytesPending = pktPtr->len + 1;
						dataPending = TRUE;
						break;
						default:;					// unhandled command
					}
				}
				else								// if controller command
				{
					buf_in(&uart_buf, pktPtr, zDataInd->nsduLength);
													// add to serial buffer
					SFRPAGE = UART1_PAGE;			// switch SFR page to set TI1
					TI1 = TRUE;						// set transmit flag
				}
				LED12_ON();							// set receive indicator LED
				rxLedCnt = LED_TIMEOUT;				// start LED timer
			}
			/* This block handles data received over the serial link
			 * 
			 * The following conditions must be met for this block to execute:
			 * 1.  A serial transmission is not in progress
			 * 2.  The serial buffer contains at least one complete packet
			 * 3.  There is no data pending transmission over Zigbee link
			 * 
			 * If the conditions are met, the packet received over the serial link
			 * is removed from serial buffer and wrapped in another packet for 
			 * retransmission.  This packet is copied to Zigbee transmit buffer and
			 * the data pending flag is set.
			 */
			SFRPAGE = UART1_PAGE;					// switch SFR page to see TI1
			if (!uart_buf.state && !dataPending && (pktLength = buf_packet(&uart_buf)))
			{
				packet xdata *pktPtr;
				bytesPending = buf_out(&uart_buf, z_buf, pktLength);
					// remove packet from serial buffer and store in Zigbee transmit buffer
				pktPtr = (packet xdata *)z_buf;
				pktPtr->addr = nodeAddr;
				pktPtr->len -= 2;
				packet_calc_crc(pktPtr);
				dataPending = TRUE;
			}
			LoopProce();
			/* This block handles transmission of data over Zigbee link
			 * 
			 * If data pending flag is set and sending is allowed, bytesPending bytes
			 * are passed to network layer for transmission.  The data pending and
			 * allowed to send flags are cleared and the transmit indicator LED (D13)
			 * is set.
			 */
			if (dataPending && canSend)
			{
				ZigbeeSendData(bytesPending);		// pass data to network layer
				dataPending = FALSE;				// clear data pending flag
				canSend = FALSE;					// clear allowed to send flag
				LED13_ON();							// set transmission indicator LED
				txLedCnt = LED_TIMEOUT;				// start LED timer
			}
		}
		/* This block does magical realtime Zigbee stuff */
		if (bitTimeIntFlagOld != bitTimeIntFlagNew)
		{
			LoopProceTimeChange();
		}
	}
	/* Leave the network if the user so desires */
	LeaveNet();
	ALL_LEDS_OFF();
}
