/*******************************************************************************
 * WJ Communications - Distributed Autonomous Reader Network
 * Zigbee device firmware
 * Zigbeehost.c - Implementation file for Zigbee networking and packet
 *	manipulation functions
 *
 * Author: Michael Walter-Echols
 * 
 * (C) WJCI 2006
 * 401 River Oaks Pkwy
 * San Jose, CA 95134-1916
 * USA
 ******************************************************************************/

#include "ZigbeeHost.h"

BYTE xdata z_buf[Z_BUF_SIZE];
BYTE xdata nodeAddr;

/*******************************************************************************
 * packet_init()
 * Initialize packet structure
 ******************************************************************************/
void packet_init(packet xdata *p)
{
	p->sof = SOF;
	p->addr = nodeAddr;
	p->len = 2;										// length 2 bytes (except SOF)
}

/*******************************************************************************
 * packet_calc_crc()
 * Calculate and attach CRC
 ******************************************************************************/
void packet_calc_crc(packet xdata *p)
{
	UINT16 crc;
	UINT8 count = p->len;
	
	p->len += 2;								// add 2 bytes for CRC
	crc = calcFwdBlockCRC(count, CRC_PRELOAD, (void *)p + 1);
													// calculate CRC from packet
	*(UINT16 *)(p->payload + count - 2) = crc;		// append to payload
}

/*******************************************************************************
 * crc_ok()
 * Check the CRC of a packet
 ******************************************************************************/
BOOL crc_ok(packet xdata *p)
{
	UINT16 crc;
	UINT8 count;

	count = p->len - 2;								// length of packet without CRC
	crc = calcFwdBlockCRC(count, CRC_PRELOAD, (void *)p + 1);
													// calculate CRC from packet
	
	return (crc == *(UINT16 *)(p->payload + count - 2));
}

/*******************************************************************************
 * CreateZigbeeCoordinator()
 * Start as Zigbee Coordinator
 ******************************************************************************/
void CreateZigbeeCoordinator(void)
{
	nwkcCoordinatorCapable = COORDINATOR_TYPE;		// set as coordinator

	if(0 == nlmeNetworkFormationRequest(SCAN_CHANNELS, SCAN_DURATION, 15, 15, PAN_ID, FALSE))
	{												// if able to form network
       nlmePermitJoiningRequest(0xFF);				// allow joining
	   LED6_ON();									// set indicator LED
	}
	else											// if unable to form network
	{
		LED13_ON();									// set error indicator LED
	}
	
}

/*******************************************************************************
 * CreateZigbeeRouter()
 * Start as Zigbee router
 ******************************************************************************/
void CreateZigbeeRouter(void)
{
    NLME_JOIN_CONFIRM *ptrNetJoinResult;

	nwkcCoordinatorCapable = ROUTER_TYPE;			// set as router

	nlmeNetworkDiscoveryRequest(SCAN_CHANNELS, SCAN_DURATION);
													// discover network
	nlmeJoinRequest(PAN_ID, TRUE, FALSE, SCAN_CHANNELS, SCAN_DURATION, POWER_MAINS, TRUE, FALSE);
													// attempt to join

    ptrNetJoinResult = (NLME_JOIN_CONFIRM*)nlmeConfirm.buffer;

	if(ptrNetJoinResult->Status == SUCCESS)			// if able to join
	{
	   nlmeStartRouterRequest(15, 15, FALSE);		// start operating as router
       nlmePermitJoiningRequest(0xFF);				// permit joining

	   LED7_ON();									// set indicator LED
	}
	else											// if unable to join
	{
       LED13_ON();									// set error indicator LED
	}
}

/*******************************************************************************
 * CreateZigbeeEndDevice()
 * Start as Zigbee end device
 ******************************************************************************/
void CreateZigbeeEndDevice(void)
{
	unsigned char shortAddr;
	unsigned long coordAddr;
    NLME_JOIN_CONFIRM *ptrNetJoinResult;

	nwkcCoordinatorCapable = END_DEVICE_TYPE;		// set as end device

	nlmeNetworkDiscoveryRequest(SCAN_CHANNELS, SCAN_DURATION);
													// discover network
	nlmeJoinRequest(PAN_ID, FALSE, FALSE, SCAN_CHANNELS, SCAN_DURATION, POWER_MAINS, TRUE, FALSE);
													// attempt to join

    ptrNetJoinResult = (NLME_JOIN_CONFIRM*)nlmeConfirm.buffer; 

	if(ptrNetJoinResult->Status == SUCCESS)			// if able to join
	{
		shortAddr = GetNetShortAddress();			// retrieve addresses
		coordAddr = GetCoordExtendedAddress();
		LED8_ON();									// set indicator LED
	}
	else											// if unable to join
	{
       LED13_ON();									// set error indicator LED
	}
}

/*******************************************************************************
 * ZigbeeSendData()
 * Pass data to network layer for transmission over Zigbee link.
 ******************************************************************************/
void ZigbeeSendData(UINT8 bydataLen)
{
	NLDE_DATA_REQUEST xdata NldeDataRequest;		// data request structure

	if (nwkcCoordinatorCapable == COORDINATOR_TYPE)	// if running as coordinator
	{
	   NldeDataRequest.dstAddr      = BROADCAST;	// address as broadcast
	}
	else											// if running as router/end device
	{
	   NldeDataRequest.dstAddr      = COORD_ADDR;	// address to coordinator
	}

   	NldeDataRequest.nsduLength      = bydataLen;	// length of frame
	NldeDataRequest.pNsdu           = z_buf;		// pointer to Zigbee buffer
	NldeDataRequest.nsduHandle      = HANDLE;		// frame handle
	NldeDataRequest.broadcastRadius = BROADCAST_RADIUS;
													// hops for broadcast to travel
	NldeDataRequest.discoverRoute   = FALSE;		// disable route discover
	NldeDataRequest.securityEnable  = FALSE;		// disable security (not implemented)
    nldeDataRequest(&NldeDataRequest);				// perform request
}

/*******************************************************************************
 * LeaveNet()
 * Leave the network if router or end device.  Dissolve the network if
 * coordinator.
 ******************************************************************************/
void LeaveNet(void)
{
    ADDRESS xdata deviceAddress;

	memset(&deviceAddress,0x00,sizeof(deviceAddress));

	nlmeLeaveRequest(deviceAddress);				// leave the network
}