/*******************************************************************************
 * WJ Communications - Distributed Autonomous Reader Network
 * Zigbee device firmware
 * ZigbeeHost.h - Header file for Zigbee networking and packet manipulation
 *	functions
 *
 * Author: Michael Walter-Echols
 * 
 * (C) WJCI 2006
 * 401 River Oaks Pkwy
 * San Jose, CA 95134-1916
 * USA
 ******************************************************************************/

#ifndef ZIGBEEHOST_H
#define ZIGBEEHOST_H

#include <string.h>									// for memcpy()
#include "../include/hal/c8051F120.h"				// SFR definitions
#include "../include/DataDef.h"						// data type definitions
#include "portdefs.h"								// LEDs and buttons
#include "serial.h"									// for packet structure definition
#include "crc.h"									// for CalcFwdBlockCRC()
#include "../include/HS_Net/HS_Net.h"				// Zigbee stack

#define Z_BUF_SIZE 		256							// Zigbee transmit buffer size
#define MAX_NODES		5							// maximum number of net nodes

/* CRC definitions */
#define CRC_TYPE 16									// 16-bit CRC
#define CRC_LEN	(CRC_TYPE / 8)						// length (in bytes) of CRC
#define CRC_PRELOAD 0xBEEF							// CRC seed

/* Zigbee network settings */
#define SCAN_CHANNELS	0x00001000L					// channel to scan
#define SCAN_DURATION	5							// time to scan a channel
#define PAN_ID			0x0002						// network identifier
#define POWER_MAINS		1							// connected to mains power
#define POWER_OTHER		0							// connected to other power source
#define BROADCAST		0xFFFF						// broadcast address
#define COORD_ADDR		0x0000						// coordinator address
#define HANDLE			1							// data handle
#define BROADCAST_RADIUS 1							// number of hops for broadcast

#define NODE_ADDRESS	0xFBF5						// memory location of node address

/* Packet manipulation functions */
/*******************************************************************************
 * packet_init()
 * Initialize a packet structure
 * 
 * Parameters:
 *	packet * - pointer to packet structure to be initialized
 * Postcondition:
 *	packet structure pointed to by parameter is reset to empty state
 *	start = 0, end = 0, and node address has been set
 ******************************************************************************/
void packet_init(packet xdata *);

/*******************************************************************************
 * packet_calc_crc()
 * Calculate and attach CRC to packet
 * 
 * Precondition:
 *	packet has been initialized
 * Parameters:
 *	packet * - pointer to packet structure to be calculated
 * Postcondition:
 *	CRC has been calculated on packet contents (excluding SOF byte), stored as
 *	the last two bytes of payload.  Packet length byte is increased by 2
 ******************************************************************************/
void packet_calc_crc(packet xdata *);

/*******************************************************************************
 * crc_ok()
 * Check the CRC of a packet
 * 
 * Precondition:
 *  packet contains CRC
 * Parameters:
 *	packet * - pointer to packet structure to be checked
 * Return value:
 *	BOOL - TRUE if calculated CRC matches CRC contained in packet, else FALSE
 * Postcondition:
 *	packet is unchanged
 ******************************************************************************/
BOOL crc_ok(packet xdata *);

/* ZigBee functions */
/*******************************************************************************
 * CreateZigbeeCoordinator()
 * Start as Zigbee coordinator
 *
 * Precondition:
 *	Node is currently not associated with any Zigbee network
 * Postcondition:
 *	If no network with ID of PAN_ID is already started, this node will act as
 *	coordinator for network with a ID of PAN_ID
 *	If network with ID of PAN_ID already exists, LED13 is lit and no network is
 *	established.
 ******************************************************************************/
void CreateZigbeeCoordinator(void);

/*******************************************************************************
 * CreateZigbeeRouter()
 * Start as Zigbee router
 *
 * Precondition:
 *	Node is currently not associated with any Zigbee network
 *	Network with ID of PAN_ID has already been established
 * Postcondition:
 *	Node is acting as router for network with ID of PAN_ID unless the join
 *	operation fails, in which case LED12 is lit and the network has not been
 *	joined
 ******************************************************************************/
void CreateZigbeeRouter(void);

/*******************************************************************************
 * CreateZigbeeEndDevice()
 * Start as Zigbee end device
 *
 * Precondition:
 *	Node is currently not associated with any Zigbee network
 *	Network with ID of PAN_ID has already been established
 * Postcondition:
 *	Node is associated with network with ID of PAN_ID unless the join operation
 *	fails, in which case LED11 is lit and the network has not been joined
 ******************************************************************************/
void CreateZigbeeEndDevice(void);

/*******************************************************************************
 * ZigbeeSendData()
 * Pass data to network layer for transmission over Zigbee link.  Address is 
 * determined by type of network association.
 * Coordinator: frame addressed to BROADCAST
 * Router/End device: frame addressed to COORD_ADDR
 * 
 * Precondition:
 *	Data to be transmitted is in z_buf
 * Parameters:
 *	UINT8 - number of bytes to transmit from z_buf
 * Postcondition:
 *	Request has been passed to network layer
 ******************************************************************************/
void ZigbeeSendData(UINT8);

/*******************************************************************************
 * Leave the network if router or end device.  Dissolve the network if
 * coordinator
 *
 * Precondition:
 *	Node is currently associated with a Zigbee network
 * Postcondition:
 *	Node is no longer associated with a Zigbee network
 ******************************************************************************/
void LeaveNet(void);

#endif
