/*******************************************************************************
 * WJ Communications - Distributed Autonomous Reader Network
 * Zigbee device firmware
 * serial.c - Implementation file for serial communications functions
 *
 * Author: Michael Walter-Echols
 * 
 * (C) WJCI 2006
 * 401 River Oaks Pkwy
 * San Jose, CA 95134-1916
 * USA
 ******************************************************************************/

#include "serial.h"

buffer xdata uart_buf;								// buffer structure for UART0

#ifdef ZIGBEE_COORDINATOR
/*******************************************************************************
 * uart0()
 * UART0 interrupt service routine.
 ******************************************************************************/
void uart0() interrupt 4 using 1
{
	register UINT8 start = uart_buf.start;			// buffer start index
	register UINT8 end = uart_buf.end;				// buffer end index

	if (RI0)										// if data received
	{
		RI0 = FALSE;								// acknowledge reception
		ES0 = FALSE;								// start of critical section
		uart_buf.end = (end + 1) % BUF_SIZE;		// increment index and wrap
		uart_buf.queue[end] = SBUF0;				// add to end of buffer
		ES0 = TRUE;									// end of critical section
	}

	if (TI0)										// if data awaiting transmission
	{
		ES0 = FALSE;								// start of critical section
		TI0 = FALSE;								// acknowledge transmit request
		if (start != end)							// while more data to send
		{
			uart_buf.state = INCONSISTANT;			// mark buffer state as inconsistant
			uart_buf.start = (start + 1) % BUF_SIZE;
													// increment index and wrap
			SBUF0 = uart_buf.queue[start];			// write data to port
		}
		else {										// if end of transmission
			uart_buf.state = CONSISTANT;			// mark buffer state as consistant
			TI0 = FALSE;							// end transmit request
		}
		
		ES0 = TRUE;									// end of critical section
	}
}

#else
/*******************************************************************************
 * uart1()
 * UART1 interrupt service routine.
 ******************************************************************************/
void uart1() interrupt 20 using 1
{
	register UINT8 start = uart_buf.start;			// buffer start index
	register UINT8 end = uart_buf.end;				// buffer end index

	if (RI1)										// if data received
	{
		RI1 = FALSE;								// acknowledge reception
		EIE2 &= 0xBF;								// start of critical section
		uart_buf.end = (end + 1) % BUF_SIZE;		// increment index and wrap
		uart_buf.queue[end] = SBUF1;				// add to end of buffer
		EIE2 |= 0x40;								// end of critical section
	}

	if (TI1)										// if data awaiting transmission
	{
		EIE2 &= 0xBF;								// start of critical section
		TI1 = FALSE;								// acknowledge transmit request
		if (start != end)							// while more data to send
		{
			uart_buf.state = INCONSISTANT;			// mark buffer state as inconsistant
			uart_buf.start = (start + 1) % BUF_SIZE;
													// increment index and wrap
			SBUF1 = uart_buf.queue[start];			// write data to port
		}
		else {										// if end of transmission
			uart_buf.state = CONSISTANT;			// mark buffer state as consistant
			TI1 = FALSE;							// end transmit request
		}

		EIE2 |= 0x40;								// end of critical section
	}
}
#endif

/*******************************************************************************
 * buf_init()
 * Initialize serial buffer
 ******************************************************************************/
void buf_init(buffer xdata *buf)
{
	buf->start = 0;
	buf->end = 0;
	buf->state = CONSISTANT;
}

/*******************************************************************************
 * buf_in()
 * Add data to serial buffer
 ******************************************************************************/
UINT8 buf_in(buffer xdata *buf, void xdata *sPtr, UINT8 count)
{
	UINT8 end = buf->end;
	INT16 act_count = buf->start - end;				// actual number of bytes
													// available in buffer
	ES0 = FALSE;									// start of critical section

	if (act_count <= 0)								// if end index wrapped
	{
		act_count += BUF_SIZE;						// adjust bytes available
	}
	act_count--;									// account for wasted byte
	
	if (count < act_count)							// if enough space available
	{												// to store desired data
		act_count = count;							// copy desired amount
	}
	
	if (end + act_count <= BUF_SIZE)				// case 1: no need to wrap
	{
		memcpy(buf->queue + end, sPtr, act_count);	// copy into buffer
		end += act_count;							// adjust end index
	}
	else											// case 2: need to wrap
	{												// around end of buffer
		memcpy(buf->queue + end, sPtr, BUF_SIZE - end);
													// copy bytes before end
		memcpy(buf->queue, sPtr + (BUF_SIZE - end), act_count - (BUF_SIZE - end));
													// copy bytes after end
		end += act_count;							// adjust end index
	}

	end %= BUF_SIZE;								// wrap index
	buf->end = end;									// update index
	ES0 = TRUE;										// end of critical section

	return act_count;								// return bytes copied
}

/*******************************************************************************
 * buf_out()
 * Remove data from serial buffer
 ******************************************************************************/
UINT8 buf_out(buffer xdata *buf, void xdata *tPtr, UINT8 count)
{
	UINT8 start = buf->start;
	INT16 act_count = buf->end - start;				// maximum number of bytes
													// available to remove
	
	if (act_count < 0)								// if end index wrapped
	{
		act_count += BUF_SIZE;						// adjust bytes available
	}
	
	if (count < act_count)							// if enough data available
	{												// to return desired amount
		act_count = count;							// adjust bytes desired
	}
	
	if (start + act_count <= BUF_SIZE)				// case 1: no need to wrap
	{												// around end of buffer
		memcpy(tPtr, buf->queue + start, act_count);
													// copy out of buffer
		start += count;								// adjust start index
	}
	else											// case 2: need to wrap
	{												// around end of buffer
		memcpy(tPtr, buf->queue + start, BUF_SIZE - start);
													// copy bytes before end
		memcpy(tPtr + (BUF_SIZE - start), buf->queue, act_count - (BUF_SIZE - start));
													// copy bytes after end
		start += act_count;							// adjust start index
	}

	start %= BUF_SIZE;								// wrap index
	buf->start = start;								// update index

	return act_count;								// return bytes copied
}

/*******************************************************************************
 * buf_packet()
 * Determine if a complete packet is waiting in buffer and if so, its length
 ******************************************************************************/
UINT8 buf_packet(buffer xdata *buf)
{
	UINT8 start = buf->start;
	UINT8 pack_len_i = (start + LEN_I) % BUF_SIZE;	// index of packet length byte
	INT16 size = buf->end - start;					// number of bytes in buffer
	BYTE pack_len = buf->queue[pack_len_i] + 1;		// length of packet candidate

	if (size < 0)									// if end index wrapped
	{
		size += BUF_SIZE;							// adjust data size
	}

	if ((size > LEN_I + 1) && (size >= pack_len))
	{	// if we have received a packet length byte and enough data specified by it
		return (pack_len);							// return length of packet
	}
	
	return 0;										// otherwise no packet
}
