/*******************************************************************************
 * WJ Communications - Distributed Autonomous Reader Network
 * Zigbee device firmware
 * serial.h - Header file for serial communications functions
 *
 * Author: Michael Walter-Echols
 * 
 * (C) WJCI 2006
 * 401 River Oaks Pkwy
 * San Jose, CA 95134-1916
 * USA
 ******************************************************************************/

#ifndef SERIAL_H
#define SERIAL_H

#include <string.h>									// for memcpy()
#include "../include/hal/c8051F120.h"				// SFR definitions
#include "../include/DataDef.h"						// data type definitions

#define BUF_SIZE 64									// size (in bytes) of serial buffer
#define CONSISTANT 0								// first byte in buffer is SOF
#define INCONSISTANT 1								// undefined buffer state

/* buffer structure definition */
typedef struct {
	BYTE queue[BUF_SIZE];							// buffer queue
	UINT8 start;									// start index
	UINT8 end;										// end index
	BOOL state;										// buffer state
} buffer;

/* packet attribute definitions */
#define MAX_REQUEST		64							// maximum length (in bytes) of request packet
#define MAX_RESPONSE	256							// maximum length (in bytes) of response packet
#define RFU				0x00						// data written to RFU byte
#define SOF				0x01						// data written to SOF byte
#define LEN_I			2							// index of length byte
#define PAYLOAD_I		3							// index of first payload byte
#define LOCAL_CMD		0xFF						// command for zigbee module

/* payload field indexes */
#define CMD				0							// command field
#define SUBCMD			1							// subcommand field
#define STATUS			0							// status byte
#define NODES_JOINED	1							// nodes joined field
#define	NODE_TABLE		2							// start of node table

/* module subcommands */
#define ECHO			0x01						// respond with success
#define NET_REPORT		0x02						// report which nodes are joined

/* packet structure definition */
typedef struct {
	BYTE sof;										// start of frame byte
	BYTE addr;										// reserved for future use
	BYTE len;										// length of packet (excluding SOF)
	BYTE payload[MAX_RESPONSE - 3];					// packet payload
} packet;

/* interrupt service routines */
/*******************************************************************************
 * UART0 interrupt service routine
 * Controls the reception and transmission of data on UART0
 *
 * Precondition:
 *	UART0 is enabled on crossbar
 *	Baud rate for UART0 has been set
 *	UART0 has been enabled to receive
 * Postcondition:
 *	Any bytes received have been added to UART0 buffer
 *	Any bytes in buffer when TI0 flag is set are sent and buffer indexes set as
 *	empty
 ******************************************************************************/
void uart0(void);

/*******************************************************************************
 * Initialize serial buffer
 *
 * Parameters:
 *	buffer xdata * - pointer to buffer to be initialized
 * Postcondition:
 *	start and end indexes for buffer have been set to 0
 ******************************************************************************/
void buf_init(buffer xdata *);

/*******************************************************************************
 * Add data to serial buffer
 * This function disables UART interrupts and can therefore block data reception
 *
 * Precondition:
 *	Buffer has been initialized
 * Parameters:
 *	buffer xdata * - pointer to buffer to add data to
 *	void xdata * - pointer to first byte of data to be added
 *	UINT8 - desired number of bytes to add
 * Return value:
 *	UINT8 - number of bytes actually added to buffer
 * Postcondition:
 *	The number of bytes indicated in return value have been added to buffer
 ******************************************************************************/
UINT8 buf_in(buffer xdata *, void xdata *, UINT8);

/*******************************************************************************
 * Remove data from serial buffer
 * This function does not disable UART interrupts and therefore does not block
 * data reception
 * 
 * Precondition:
 *	Buffer has been initialized
 * Parameters:
 *	buffer xdata * - pointer to buffer to remove data from
 *	void xdata * - pointer to location to copy data to
 *	UINT8 - desired number of bytes to remove
 * Return value:
 *	UINT8 - number of bytes actually removed from buffer
 * Postcondition:
 *	The number of bytes indicated in return value have been copied to the
 *	pointed to location
 ******************************************************************************/
UINT8 buf_out(buffer xdata *, void xdata *, UINT8);

/*******************************************************************************
 * Determine if a complete packet is waiting in buffer and if so, its length
 * 
 * Precondition:
 *	Buffer has been initialized
 * Parameters:
 *	buffer xdata * - pointer to buffer to check
 * Return value:
 *	UINT8 - length (in bytes) of packet in buffer, 0 if none/incomplete
 * Postcondition:
 *	A packet of length indicated by the return value is in buffer
 *	Buffer is unchanged
 ******************************************************************************/
UINT8 buf_packet(buffer xdata *);

#endif
